/*
 * Layers.h
 *
 *  Created on: Nov 28, 2012
 *      Author: sbak
 */

#ifndef LAYERS_H_
#define LAYERS_H_

#include "common.h"

#include <map>

#include "ltibak/ltiGradientFunctor.h"
#include "ltibak/ltiMatrix.h"

#include "ltibak/ltiConvolution.h"
#include "ltibak/ltiGaborKernels.h"
#include "ltibak/ltiLaplacianKernel.h"
#include "ltibak/ltiGaussKernels.h"

#define LAYER_X 						0
#define LAYER_Y 						(LAYER_X + 1)
#define LAYER_RED 						(LAYER_Y + 1)
#define LAYER_GREEN						(LAYER_RED + 1)
#define LAYER_BLUE						(LAYER_GREEN + 1)
#define LAYER_Magnitude_R				(LAYER_BLUE + 1)
#define LAYER_Orientation_R				(LAYER_Magnitude_R + 1)
#define LAYER_Magnitude_G				(LAYER_Orientation_R + 1)
#define LAYER_Orientation_G				(LAYER_Magnitude_G + 1)
#define LAYER_Magnitude_B				(LAYER_Orientation_G + 1)
#define LAYER_Orientation_B				(LAYER_Magnitude_B + 1)
#define LAYER_INTENSITY					(LAYER_Orientation_B + 1)
#define LAYER_Magnitude_INTENSITY		(LAYER_INTENSITY + 1)
#define LAYER_Orientation_INTENSITY		(LAYER_Magnitude_INTENSITY + 1)
#define LAYER_DISTANCE					(LAYER_Orientation_INTENSITY + 1)
#define LAYER_GABOR_0					(LAYER_DISTANCE + 1)
#define LAYER_GABOR_90					(LAYER_GABOR_0 + 1)
#define LAYER_GABOR_45					(LAYER_GABOR_90 + 1)
#define LAYER_GABOR_135					(LAYER_GABOR_45 + 1)
#define LAYER_LAPLACIAN					(LAYER_GABOR_135 + 1)
#define LAYER_GAUSSIAN					(LAYER_LAPLACIAN + 1)

//number of implemented layers
#define MAX_NLAYER (LAYER_GAUSSIAN + 1)

class Layer {

public:

	Layer(){
		index = -1;
		name = "NULL";
	}

	Layer(int _index, std::string _name, CvRect _rect){
		index = _index;
		name = _name;
		rect = _rect;
		values.resize(rect.height, rect.width, (float) 0);
	}

	Layer(const Layer& l){
		index = l.index;
		name = l.name;
		rect = l.rect;
		values = lti::channel(l.values);
	}

	int index;
	lti::channel values;
	std::string name;
	CvRect rect;

	float& at (int row, int column){
		return values.at(row,column);
	}

	std::string getKey(){
		return getKey(index, rect);
	}

	static std::string getKey(uint _index, CvRect& rect){
			std::string key = IntToString(_index) + ";" + IntToString(rect.x)
						+ ";" + IntToString(rect.y) + ";" + IntToString(rect.width)
						+ ";" + IntToString(rect.height);
			return key;
	}

	void display(){
		IplImage* tmp = cvCreateImage(cvSize(values.columns(), values.rows()),IPL_DEPTH_8U, 1);
		float maxValue = 0;

		for (int i = 0; i < values.rows(); i++) {
			for (int j = 0; j < values.columns(); j++) {
				if (values.at(i, j) > maxValue)
						maxValue = values.at(i, j);
			}
		}

		float factor = 255.0f / float(maxValue);

		for (int i = 0; i < values.rows(); i++) {
			for (int j = 0; j < values.columns(); j++) {
				int px = j;
				int py = i;
				(tmp->imageData + tmp->widthStep * py)[px] = values.at(i, j) * factor;
			}
		}
		show_img(tmp, (name + " " + FloatToString(maxValue)).c_str(), values.columns() * 5, values.rows() * 5, false);
	}
};

class Layers {

	IplImage* image;
	std::map<std::string, Layer> layers;

public:
	Layers(IplImage* img);
	Layers& operator=(const Layers& l);
	Layers(const Layers& l);
	Layers* clone();

	virtual ~Layers();
	void printSelectedLayers();
	Layer getLayer(uint index, CvRect* rect = NULL);
	IplImage* getImage();

private:

	Layer getX(CvRect& rect);
	Layer getY(CvRect& rect);
	Layer getRed(CvRect& rect);
	Layer getGreen(CvRect& rect);
	Layer getBlue(CvRect& rect);
	Layer getGradient_Magnitude_R(CvRect& rect);
	Layer getGradient_Orientation_R(CvRect& rect);
	Layer getGradient_Magnitude_G(CvRect& rect);
	Layer getGradient_Orientation_G(CvRect& rect);
	Layer getGradient_Magnitude_B(CvRect& rect);
	Layer getGradient_Orientation_B(CvRect& rect);
	Layer getI(CvRect& rect);
	Layer getGradient_Magnitude_I(CvRect& rect);
	Layer getGradient_Orientation_I(CvRect& rect);
	Layer getDistance(CvRect& rect);
	Layer getGabor_0(CvRect& rect);
	Layer getGabor_90(CvRect& rect);
	Layer getGabor_45(CvRect& rect);
	Layer getGabor_135(CvRect& rect);
	Layer getLaplacian(CvRect& rect);
	Layer getGaussian(CvRect& rect);

	void addLayer(Layer l);
	Layer _getLayer(uint index, CvRect& rect);

	void gradient(lti::channel& src, lti::channel& mag, lti::channel& arg);
	void gabor(lti::channel& src, lti::channel& res, const int& size,
			const double& theVariance, const double& frequency,
			const double& angle);
	void laplacian(lti::channel& src, lti::channel& res);
	void gaussian(lti::channel& src, lti::channel& res);
};

#endif /* LAYERS_H_ */
