/*
 * MTS.h
 *
 *  Created on: Dec 17, 2010
 *      Author: sbak
 */

#ifndef MTS_H_
#define MTS_H_

#include "common.h"
#include "CovarianceMatrix.h"
#include "SignatureInterface.h"
#include "FeatureInterface.h"
#include "SignatureModel.h"
#include <qdatetime.h>
#include <qstring.h>

#define MTS_DEBUG_TIME
#define IMG_FORMAT "png"


class MTS :public SignatureInterface {

	std::string ID;
	std::string source_link;
	QDateTime start_t;			//time when signature has been created
	QDateTime end_t;			//time when signature has been updated for the last time
	IplImage* i_image;			//input image (the last image used for signature extraction)
	IplImage* p_image;			//scaled and normalized image (after processing)
	int nframes;

	//MODEL INFO
	SignatureModel* model;
	std::vector<bool> _integrals;

	std::vector<FeatureInterface*> features;
	std::vector<float> getNormalizedWeigths();
	FeatureInterface* build(FeatureInterface* feature, CovarianceMatrix& engine);
	FeatureInterface* getInstance(std::string);


public:

	MTS();
	MTS(std::string src_link);
	MTS& operator=(const MTS& s);
	MTS(const MTS& m);
	float operator-(const MTS s);
	bool operator!=(const MTS s);

	virtual ~MTS();

	void create(std::string ID, IplImage* img, std::string config = "config.xml");
	void update(IplImage* _img);
	float getDistance(MTS sig);
	float getRDistance(MTS sig);

	void releaseImages();

	//GUI functions
	std::string getID();
	void draw(IplImage* img, CvRect roi = cvRect(0,0,0,0));
	void drawROI(IplImage* img, CvRect roi = cvRect(0,0,0,0));
	IplImage* getInputImage();
	IplImage* getProcessedImage();
	std::vector<float> getDistanceVector(MTS& sig);

	float getDistance(SignatureInterface* sig);


	std::vector<float> serialize();
	IplImage* getDisplayImage(){return getInputImage();};
	IplImage* getDebugImage(){return getProcessedImage();};
	void display(IplImage* img, CvRect roi){drawROI(img, roi);};
	std::string getSource(){return source_link;};
	void save(std::string fileName);
	bool load(std::string fileName);
};

#endif /* MTS_H_ */
