/*
 * SignatureFactory.h
 *
 *  Created on: May 14, 2013
 *      Author: sbak
 */

#ifndef SIGNATUREFACTORY_H_
#define SIGNATUREFACTORY_H_

#include "SignatureInterface.h"
#include "qt_support.h"
#include "MTS.h"

#define SIG_ERROR -1
#define SIG_MTS 0

class SignatureFactory {

public:
	SignatureFactory() {
	}
	;

	virtual ~SignatureFactory();

	static int getType(std::string filename){

		QFile file((filename + ".xml").c_str());

		if (!file.open(QIODevice::ReadOnly)) {
			_ERROR("file.open( " + filename + " ) crashed!");
			return SIG_ERROR;
		}

		QDomDocument doc;
		if (!doc.setContent(&file)) {

			_ERROR("doc.setContent( " + filename + " ) crashed!");
			file.close();
			return SIG_ERROR;
		}

		QDomElement root = doc.documentElement();

		if (root.tagName() == "MTS") {
			file.close();
			return SIG_MTS;
		}else{
			_ERROR("Signature [" + root.tagName().toStdString() + "] is not recognized!");
			file.close();
			return SIG_ERROR;
		}
	}

	static SignatureInterface* getSignature(int type) {

		switch (type) {
		case SIG_MTS:
			return new MTS();
		default:
			_ERROR("FACTORY cannot produce specified signature type: " + IntToString(type));
			return NULL;
		}
	}
};

#endif /* SIGNATUREFACTORY_H_ */
