/*
 * mtsParameters.h
 *
 *  Created on: Feb 6, 2013
 *      Author: sbak
 */

#ifndef MTSPARAMETERS_H_
#define MTSPARAMETERS_H_

#include "common.h"
#include "CovarianceMatrix.h"
#include "SignatureModel.h"

#include <sys/time.h>

#include <QFile>

//MODEL parameters
int mts_equalization;
float mts_resolution;
int mts_use_integrals;
std::string mts_model;

time_t getTimeStamp(){

	return time(NULL);
}

QDomElement saveParameters(){

	QDomDocument doc("NULL");
	QDomElement param = doc.createElement("PARAMETERS");
	param.setAttribute("MODEL", mts_model.c_str());
	param.setAttribute("USE_INTEGRALS", mts_use_integrals);
	param.setAttribute("EQUALIZATION", mts_equalization);
	param.setAttribute("RESOLUTION", mts_resolution);
	return param;
}

void mts_saveParameters(std::string filename) {

	QDomDocument doc("MTS_CONFIG");
	QDomElement root = saveParameters();
	doc.appendChild(root);

	QFile file((filename).c_str());
	if (!file.open(QIODevice::WriteOnly)) {
		return;
	}

	QTextStream ts(&file);
	ts << doc.toString();
	file.close();
}

void loadParameters(QDomElement root){

	mts_model =  root.attribute("MODEL","NULL").toStdString();
	mts_use_integrals = root.attribute("USE_INTEGRALS", "1").toInt();
	mts_equalization = root.attribute("EQUALIZATION", "1").toInt();
	mts_resolution = root.attribute("RESOLUTION", "0.2").toFloat();
}

bool mts_loadParameters(std::string filename){

	QFile file((filename).c_str());
	if (!file.open(QIODevice::ReadOnly)) {

		_ERROR("file.open( " + filename + " ) crashed!");
		return false;
	}
	QDomDocument doc;
	if (!doc.setContent(&file)) {

		_ERROR("doc.setContent( " + filename + " ) crashed!");
		file.close();
		return false;
	}
	QDomElement root = doc.documentElement();
	if (root.tagName() != "PARAMETERS") {
		_ERROR("BAD XML file!");
		file.close();
		return false;
	}

	loadParameters(root);

	file.close();

	return true;
}

IplImage* preprocessIMG(IplImage* input, SignatureModel* model){


	CvSize human_size = cvSize(model->width, model->height);
	IplImage* output = cvCreateImage(human_size, input->depth, input->nChannels);
	IplImage* cropped = getRectImage(input, mts_resolution);

	if(mts_equalization){
		cvEqualize3Hist(cropped, cropped);
	}
	cvResize(cropped, output);
	cvReleaseImage(&cropped);

	return output;
}




#endif /* MTSPARAMETERS_H_ */
