#ifndef _RLEBLOB_H_
#define _RLEBLOB_H_

#include <stdio.h>
#include <stdlib.h>

#include "rectangle.h"
#include "parallelepiped.h"
#include <list>
#include <string>

// #include "sup/CSmartPointer.h"
#include <tr1/memory>

namespace vsip {

//Forward declaration

struct scene_model;

/** Maximal number of connected regions in a blobs */
#define BLOB_MAX_REGIONS	5000
#define BL_INFINITY             MAXDOUBLE

//for Bernard posture
#define POSTURE_RESULTS_NUMBER 5

/**
 * type of blob. A blob will have one of these types after being classified.
 * @note DON'T CHANGE order of declaration.
 * @note OR, if you change it (and you know what you are doing!) please change
 * @note the order of colors in _blob_RGBcolor matrix in blob.c
 */

enum blob_type_t
{
  UNKNOWN      = 0,	/**< blob type unknown                           */
  CROWD        = 1,	/**< blob contains many many persons             */
  PERSON_GROUP = 2, /**< may be a person, maybe a group              */
  A_GROUP      = 3, /**< blob is a group                             */
  PERSON       = 4,	/**< blob is a person                            */
  CONTX_OBJECT = 5, /**< blob is a context object, ex: the door      */
  VEHICLE      = 6,	/**< blob is a vehicle, ex: metro, car           */
  NOISE        = 7,	/**< noise                                       */
  VALIDATION   = 8,	/**< blob unknown found in zone of TBL, for RATP */
  CAR          = 9,
  TRUCK        = 10,
  MOTORCYCLE   = 11,
  PART_OF_GROUP        = 12, /**< blob is a part of a group                   */
  PART_OF_PERSON_GROUP = 13, /**< blob is a part of a person_group            */
  UNDETERMINATE        = 14,  /**< blob type is indeterminate                  */
  CHILD                = 15,
  PARKED_VEHICLE       = 16,
  FACE       = 17,
  TWOPERSONS = 18,
  THREEPERSONS = 19,
  PART_OF_PERSON =20,         /**<blob is a part of person  */
  PERSON_BENDING =21,          /**< blob is a bending position of a person       */

// gerhome
  LEFT_FOOD_CUPBOARD_DOOR =22,          /**< blob is a opening position of a cupboard door       */
  RIGHT_FOOD_CUPBOARD_DOOR =23,          /**< blob is a opening position of a cupboard door       */

  LEFT_DISH_CUPBOARD_DOOR =24,          /**< blob is a opening position of a cupboard door       */
  RIGHT_DISH_CUPBOARD_DOOR =25,          /**< blob is a opening position of a cupboard door       */

  CHAIR =26,          /**< blob is a position of a chair       */
  FRIDGEDOOR =27,          /**< blob is a position of a fridge door       */
  MICROWAVE_DOOR =28,          /**< blob is a position of a microwave door       */
  LUGGAGE =29,			/** blob is a luggage */

  ANY2D = 30,                   /** For learning context: Any class even UNKNOWN*/
  ANY3D = 31,                    /** For learning context: Any class without UNKNOWN*/
  CARGOVEHICLE = 32              /**Apron cargo vehicle.*/
// nadia

};

// ajoute le 20/08/2007 par Nadia pour la sub-type definit par Marcos
enum blob_subtype_t
{
  ST_UNKNOWN      = 0,	/**< blob subtype unknown       */
  ST_UNDEFINED    = 1,	/**< blob subtype undefined        */
  ST_NO_SUBTYPES  = 2,	/**< constant for rigid objects */
  ST_STANDING     = 3,	/**< Standing posture           */
  ST_CROUCHING    = 4,	/**< Crouching posture          */
  ST_LYING        = 5,	/**< Lying posture              */
  ST_ARMUP        = 6, /**< Standing posture with arm(s) up*/
};

typedef enum blob_type_t blob_type_t;
typedef enum blob_subtype_t blob_subtype_t;

//

//Coding by binary flags for blob occlusion to allow the representation of multiple types of occlusion at the same time (Marcos Zuniga)
typedef enum
{
  MM_OCCL_NONE             = 0x00000000,//00000000000000000000000000000000,  // no occlusion type
  MM_CAM_OCCL_BOTTOM       = 0x00000001,//00000000000000000000000000000001,  // only occlusion on image bottom
  MM_CAM_OCCL_TOP          = 0x00000002,//00000000000000000000000000000010,  // only occlusion on image top
  MM_CAM_OCCL_LEFT         = 0x00000004,//00000000000000000000000000000100,  // only occlusion on image left
  MM_CAM_OCCL_RIGHT        = 0x00000008,//00000000000000000000000000001000,  // only occlusion on image right
  MM_CAM_OCCL_BOTTOM_LEFT  = 0x00000005,//00000000000000000000000000000101,  // occlusion on bottom and left corner of image
  MM_CAM_OCCL_BOTTOM_RIGHT = 0x00000009,//00000000000000000000000000001001,  // occlusion on bottom and right corner of image
  MM_CAM_OCCL_TOP_LEFT     = 0x00000006,//00000000000000000000000000000110,  // occlusion on top and left corner of image
  MM_CAM_OCCL_TOP_RIGHT    = 0x0000000A,//00000000000000000000000000001010,  // occlusion on top and right corner of image
  MM_CAM_OCCL_MASK         = 0x0000000F,//00000000000000000000000000001111,  // camera occlusion mask
  MM_OBJECT_BOTTOM         = 0x00000010,//00000000000000000000000000010000,  // blob is occluded on bottom part by a context object
  MM_OBJECT_TOP            = 0x00000020,//00000000000000000000000000100000,  // blob is occluded on top part by a context object
  MM_OBJECT_LEFT           = 0x00000040,//00000000000000000000000001000000,  // blob is occluded on left part by a context object
  MM_OBJECT_RIGHT          = 0x00000080,//00000000000000000000000010000000,  // blob is occluded on right part by a context object
  MM_OBJECT_BOTTOM_LEFT    = 0x00000050,//00000000000000000000000001010000,  // blob is occluded on bottom-left part by a context object
  MM_OBJECT_BOTTOM_RIGHT   = 0x00000090,//00000000000000000000000010010000,  // blob is occluded on bottom-right part by a context object
  MM_OBJECT_TOP_LEFT       = 0x00000060,//00000000000000000000000001100000,  // blob is occluded on top-left part by a context object
  MM_OBJECT_TOP_RIGHT      = 0x000000A0,//00000000000000000000000010100000,  // blob is occluded on top-right part by a context object
  MM_OBJECT_OCCL_MASK      = 0x000000F0,//00000000000000000000000011110000,  // context object occlusion mask
  MM_DYNAMIC_BOTTOM        = 0x00000100,//00000000000000000000000100000000,  // blob is occluded on bottom part by a mobile object
  MM_DYNAMIC_TOP           = 0x00000200,//00000000000000000000001000000000,  // blob is occluded on top part by a mobile object
  MM_DYNAMIC_LEFT          = 0x00000400,//00000000000000000000010000000000,  // blob is occluded on left part by a mobile object
  MM_DYNAMIC_RIGHT         = 0x00000800,//00000000000000000000100000000000,  // blob is occluded on right part by a mobile object
  MM_DYNAMIC_BOTTOM_LEFT   = 0x00000500,//00000000000000000000010100000000,  // blob is occluded on bottom-left part by a mobile object
  MM_DYNAMIC_BOTTOM_RIGHT  = 0x00000900,//00000000000000000000100100000000,  // blob is occluded on bottom-right part by a mobile object
  MM_DYNAMIC_TOP_LEFT      = 0x00000600,//00000000000000000000011000000000,  // blob is occluded on top-left part by a mobile object
  MM_DYNAMIC_TOP_RIGHT     = 0x00000A00,//00000000000000000000101000000000,  // blob is occluded on top-right part by a mobile object
  MM_DYNAMIC_OCCL_MASK     = 0x00000F00,//00000000000000000000111100000000,  // dynamic occlusion mask
  MM_HORIZONTAL_OCCL_MASK  = 0x000000CC,//00000000000000000000000011001100,  // horizontal static occlusion mask
  MM_VERTICAL_OCCL_MASK    = 0x00000033,//00000000000000000000000000110011,  // vertical static occlusion mask
  MM_HORIZONTAL_DYN_MASK   = 0x00000CCC,//00000000000000000000110011001100,  // horizontal dynamic occlusion mask
  MM_VERTICAL_DYN_MASK     = 0x00000333,//00000000000000000000001100110011,  // vertical dynamic occlusion mask
  MM_BOTTOM_OCCL           = 0x00000111,//00000000000000000000000100010001,  // bottom occlusion
  MM_TOP_OCCL              = 0x00000222,//00000000000000000000001000100010,  // top occlusion
  MM_LEFT_OCCL             = 0x00000444,//00000000000000000000010001000100,  // left occlusion
  MM_RIGHT_OCCL            = 0x00000888,//00000000000000000000100010001000,  // right occlusion
  MM_TOTAL_OCCLUSION       = 0x00010000,//00000000000000010000000000000000   // total occlusion of an object
  MM_OBJECT_LOST           = 0x00020000,//00000000000000100000000000000000,  // blob is just an estimation based on previously obtained information, its data is unreliable
  MM_NOT_VISIBLE_MASK      = 0x00030000,//00000000000000110000000000000000,  // not visible blob, lost or totally occluded
  MM_PARTIALLY_DETECTED    = 0x00040000,//00000000000001000000000000000000   // total occlusion of an object
  MM_PART_OF_BIGGER        = 0x00080000,//00000000000010000000000000000000   // total occlusion of an object
  MM_NOT_3D_TRACKABLE_MASK = 0x000C0000,//00000000000011000000000000000000,  // not visible blob, lost or totally occluded
  MM_AOI_OUTSIDE           = 0x02000000,//00000010000000000000000000000000,  // blob is outside aoi ground
  MM_AOI_REINSIDE          = 0x04000000,//00000100000000000000000000000000,  // blob is re-inside aoi ground
  MM_OBJECT_ON_TOP         = 0x08000000,//00001000000000000000000000000000,  // blob is 10 pixels (+/-) above object
  MM_OBJECT_INSIDE         = 0x10000000,//00010000000000000000000000000000,  // blob is inside object
  MM_OBJECT_SIT            = 0x20000000,//00100000000000000000000000000000,  // blob is sitting on chair
  MM_BLOB_ON_VEHICLE_AOI   = 0x40000000,//01000000000000000000000000000000,  // blob is inside vehicle aoi
  MM_CAM_TOP_VIEW          = 0x80000000 //10000000000000000000000000000000,  // blob is inside vehicle aoi
} blob_occl_type;

struct density_data{
  int TP, TN, FP, FN;
  int npixels;
  double hrate, vrate;
  //Added for tracking
  int ncols;
};

typedef struct density_data *ddata_t;

typedef struct prob_rel_data *prdata_t;

//Structure to associate a blob type to a certain stimation of the

struct prob_rel_data{
  blob_type_t 	 type; 		// blob type
  blob_subtype_t subtype;	// subtype for postural objects added by marcos
  blob_occl_type   occl_type;                            /**< occlusion type                               */

  struct rectangle bbox;		                 /**< 2D bounding box dimensions possibly modified by static occlusion resolution */

  struct parallelepiped _3Dbbox;		         /**< bounding box 	                               */
  struct density_data ddata;    //Data associated to fitness of 3D bounding box according to moving pixels.
  double w, l, h;               //Dimensions for the 3d bounding box.
  double position3d[3];         /**< 3D position of blob for a given model */
  double Rw, Rl, Rh;            //Reliability measures for each dimension.
  double Pw, Pl, Ph;            //Model probability for each dimension.
  double alpha;                 /**< Orientation angle (radians) between 3D projection on height 0 of BOTTOM line
				     of 2D bbox and line formed by points (x[1],y[1]) and (x[3],y[3]) of 3D bbox.*/
  double P;                     //Basic Probability that the blob belongs to the class
  double PR;                    //Reliable Probability that the blob belongs to the class
  double R;                     //Global Reliability of the 3d dimensions of the blob
  double DP;                    //Probability measure considering 3D dimensional proportions.

  prdata_t subtypes_list;       //List of subtypes for current type.
  prdata_t next;                //Next data element in list;
  prdata_t previous;            //Previous data element in list;

};

// for Bernard posture
typedef struct {
	int posture;
	double orientation;
	double error;
} posture_results;
//

/** a blob_t is pointer to a blob structure. */
typedef std::tr1::shared_ptr < struct RLEblob > blob_t;

/**
 * blob structure.
 */
struct RLEblob
{
  int 		     ident;		                 /**< identificator	                               */
  blob_occl_type   occl_type;                            /**< occlusion type                               */
  blob_subtype_t subtype;                                /**< Subtype for postural object types */
  struct rectangle bbox;		                 /**< bounding box 	                               */
  struct parallelepiped _3Dbbox;		         /**< bounding box 	                               */
  struct density_data ddata;                             /**< Data associated to fitness of 3D bounding box according to moving pixels. */
  int *moving_pixels;                                    /**< List that could contain the moving pixels of a blob. */
							 /**< It is an array represented as a list, with moving pixels = 1, and background pixels = 0.*/
                                                         /**< Used in tracking for difered pixel analysis.*/
  double 		     width3d;	                 /**< 3D width  		                           */
  double 		     height3d;	                 /**< 3D height  	                               */
  double 		     length3d;	                 /**< 3D length  	                               */
  double                     alpha;                      /**< Orientation angle (radians) between 3D X Axis and line formed by
							      points (x[1],y[1]) and (x[3],y[3]) of 3D bbox.*/
  int position;
/* Positions: (Position 4 is Middle-Center and correspond when blob is absolutely between focal-point and ground)
               Left-Front(0)     Middle-Front(1)    Right-Front(2)
                            \           |          /
                               \        |       /
                                  \     |    /
               Left-Center(3)<-----Focal Point----->Right-Center(5)
                                  /     |    \
                               /        |       \
                            /           |          \
	       Left-Back(6)       Middle-Back(7)    Right-Back(8)
  */

  int                        occlusion;                  /** 1 if contained solution in blob corresponds to a solution of the occlusion list
							     0 if contained solution in blob corresponds to a solution of the normal list **/
  double                     beta;                       /** Angle (radians) between 3D projection on height 0 of
							     CENTER 2D Y of blob line and 3D X axis.*/
  int                        beta_direction;             /** +1: if positive growth of angle in image plane implies positive growth of angle
							         in world xy plane.
							     -1: if positive growth of angle in image plane implies negative growth of angle
							         in world xy plane.
							 */
  double                     gP;                         /** Global probability of belonging to the class. */
  double                     gPR;                        /** Reliable Probability that the blob belongs to the class. */
  double                     gR;                         /** Mean reliability of stimated 3D model. */
  double                     gDP;                        /** Probability measure considering 3D dimensional proportions. */

  /**< Dimensions Order: [width3d, height3d, length3d].*/
  double R[3];            /**< Reliability measures for each dimension.*/
  double P[3];            /**< Model probability for each dimension.*/

  blob_type_t 	 type; 		                 /**< blob type                                    */
  int    		     xgrav;		                 /**< x coordinate of the center of gravity        */
  int    		     ygrav;		                 /**< y coordinate of the center of gravity        */
  int    		     num_pix;	                 /**< number of pixels                             */
  double	         position3d[3];              /**< 3D position of blob           */
  int   		     regions[BLOB_MAX_REGIONS];  /**< array of identificators of component regions */
  int	   		     nmb_regions;	             /**< number of component regions                  */
  double 		     moments[2][2];	             /**< momemts                                      */
  int	   		     vehicle_aoi;	             /**< 1 if (xgrav, ygrav) inside a vehicle ROI     */
  int	      proba_class[8];
  std::tr1::shared_ptr < struct RLEblob > next;		                 /**< next blob in list                            */
  std::tr1::shared_ptr < struct RLEblob > previous;	                 /**< previous blob in list                        */

  prdata_t normal_prdata;                        /**< list of best 3d associations for each model */
  prdata_t occ_prdata;                           /**< list of best 3d associations for each model, taking into account the posibility of
						    occlusion. */
  prdata_t best_prdata;                        /**< pointer to best 3d model association */

  int reduced_size;                              /*1 IF BLOB TO LITTLE TO FIT WITH ANY OF THE EXISTING PRE-DEFINED MODELS, ELSE 0 */

  int excesive_size;                             /*1 IF BLOB TO LITTLE TO FIT WITH ANY OF THE EXISTING PRE-DEFINED MODELS, ELSE 0 */

  int                       frameNumber;            /**< Frame Number where blob was processed. */
  int       currentTimeDiffMilliSeconds;            /**< Time difference in milliseconds between previous frame and frame where blob was processed. */

// for Bernard posture
  int m_3dposture; /**< posture of the blob if it is a PERSON, -1 else. */
  double m_3dorientation; /**<orientation of the blob if it is a PERSON, -1 else. */
  posture_results m_posture_results[POSTURE_RESULTS_NUMBER]; /**<gives the POSTURE_RESULTS_NUMBER best recogniton for the posture (the posture, the orientation and the error) */

};

/** a blob_t is pointer to a blob structure. */
typedef std::tr1::shared_ptr < struct RLEblob > blob_t;

/** @{ */
/** macro for getting blob attribut. */
#define BLOB_IDENT(_bl)		((_bl)->ident)

#define BLOB_FRAME_NUMBER(_bl)		((_bl)->frameNumber)
#define BLOB_TIME_DIFF_MSEC(_bl)	((_bl)->currentTimeDiffMilliSeconds)

#define BLOB_IS_REDUCED_SIZE(_bl)       ((_bl)->reduced_size)
#define BLOB_IS_EXCESIVE_SIZE(_bl)      ((_bl)->excesive_size)

#define BLOB_BBOX(_bl)		(&((_bl)->bbox))
#define BLOB_3DBBOX(_bl)        (&((_bl)->_3Dbbox))
#define BLOB_DDATA(_bl)		(&((_bl)->ddata))
#define BLOB_MOVING_PIXELS(_bl)	((_bl)->moving_pixels)

#define DDATA_TP(_dd)		(_dd)->TP
#define DDATA_TN(_dd)		(_dd)->TN
#define DDATA_FP(_dd)		(_dd)->FP
#define DDATA_FN(_dd)		(_dd)->FN
#define DDATA_NPIX(_dd)		(_dd)->npixels
#define DDATA_HRATE(_dd)	(_dd)->hrate
#define DDATA_VRATE(_dd)	(_dd)->vrate
#define DDATA_COLS_NUM(_dd)	(_dd)->ncols

#define BLOB_3DBBOX_H(_bl)	  (PARALL_H(BLOB_3DBBOX(_bl)))
#define BLOB_3DBBOX_X_i(_bl, _i)  (PARALL_X_i(BLOB_3DBBOX(_bl), _i))
#define BLOB_3DBBOX_Y_i(_bl, _i)  (PARALL_Y_i(BLOB_3DBBOX(_bl), _i))
#define BLOB_3DBBOX_POINT_i(_bl, _i)  (PARALL_POINT_i(BLOB_3DBBOX(_bl), _i))

#define BLOB_3DBBOX_X2D_BASE_i(_bl, _i) (PARALL_X2D_BASE_i(BLOB_3DBBOX(_bl), _i))
#define BLOB_3DBBOX_Y2D_BASE_i(_bl, _i) (PARALL_Y2D_BASE_i(BLOB_3DBBOX(_bl), _i))
#define BLOB_3DBBOX_2D_BASE_POINT_i(_bl, _i) (PARALL_2D_BASE_POINT_i(BLOB_3DBBOX(_bl), _i))

#define BLOB_3DBBOX_X2D_H_i(_bl, _i) (PARALL_X2D_H_i(BLOB_3DBBOX(_bl), _i))
#define BLOB_3DBBOX_Y2D_H_i(_bl, _i) (PARALL_Y2D_H_i(BLOB_3DBBOX(_bl), _i))
#define BLOB_3DBBOX_2D_H_POINT_i(_bl, _i) (PARALL_2D_H_POINT_i(BLOB_3DBBOX(_bl), _i))

#define BLOB_POSITION(_bl)      ((_bl)->position)

#define BLOB_XLEFT(_bl)		(RECT_XLEFT(BLOB_BBOX(_bl)))
#define BLOB_X(_bl)		(BLOB_XLEFT(_bl))
#define BLOB_XRIGHT(_bl)	(RECT_XRIGHT(BLOB_BBOX(_bl)))
#define BLOB_YTOP(_bl)		(RECT_YTOP(BLOB_BBOX(_bl)))
#define BLOB_Y(_bl)		(BLOB_YTOP(_bl))
#define BLOB_YBOTTOM(_bl)	(RECT_YBOTTOM(BLOB_BBOX(_bl)))

#define BLOB_WIDTH(_bl)		(RECT_WIDTH(BLOB_BBOX(_bl)))
#define BLOB_HEIGHT(_bl)	(RECT_HEIGHT(BLOB_BBOX(_bl)))

#define BLOB_XCENTER(_bl)	(BLOB_XLEFT(_bl) + BLOB_WIDTH(_bl)/2)
#define BLOB_YCENTER(_bl)	(BLOB_YTOP(_bl) + BLOB_HEIGHT(_bl)/2)

#define BLOB_TYPE(_bl)		((_bl)->type)
#define BLOB_SUBTYPE(_bl)	((_bl)->subtype)
#define BLOB_OCCL_TYPE(_bl) ((_bl)->occl_type)

#define BLOB_3D_WIDTH(_bl)	((_bl)->width3d)
#define BLOB_3D_HEIGHT(_bl)	((_bl)->height3d)
#define BLOB_3D_LENGTH(_bl)	((_bl)->length3d)

#define BLOB_PW(_bl)	((_bl)->P[0])
#define BLOB_PH(_bl)	((_bl)->P[1])
#define BLOB_PL(_bl)	((_bl)->P[2])
#define BLOB_RW(_bl)	((_bl)->R[0])
#define BLOB_RH(_bl)	((_bl)->R[1])
#define BLOB_RL(_bl)	((_bl)->R[2])

#define BLOB_ALPHA(_bl)	((_bl)->alpha)
#define BLOB_BETA(_bl)	((_bl)->beta)
#define BLOB_BETA_DIRECTION(_bl)	((_bl)->beta_direction)


#define BLOB_P(_bl)	((_bl)->gP)
#define BLOB_PR(_bl)	((_bl)->gPR)
#define BLOB_R(_bl)	((_bl)->gR)
#define BLOB_DP(_bl)	((_bl)->gDP)

#define BLOB_OCCLUSION(_bl)     ((_bl)->occlusion)

#define BLOB_PRD_NORMAL(_bl)	((_bl)->normal_prdata)
#define BLOB_PRD_OCC(_bl)	((_bl)->occ_prdata)
#define BLOB_PRD_BEST(_bl)	((_bl)->best_prdata)

//for Bernard posture
#define BLOB_3D_POSTURE(_bl) ((_bl)->m_3dposture)
#define BLOB_3D_ORIENTATION(_bl) ((_bl)->m_3dorientation)
#define BLOB_HISTORY_POSTURE(_bl,i) ((_bl)->m_posture_results[i])
//
#define BLOB_XGRAV(_bl)		((_bl)->xgrav)
#define BLOB_YGRAV(_bl)		((_bl)->ygrav)
#define BLOB_NUM_PIX(_bl)	((_bl)->num_pix)
#define BLOB_3D_X(_bl)		((_bl)->position3d[0])
#define BLOB_3D_Y(_bl)		((_bl)->position3d[1])
#define BLOB_3D_Z(_bl)		((_bl)->position3d[2])
#define BLOB_3D_POSITION(_bl)	((_bl)->position3d)
#define BLOB_REGIONS(_bl)	((_bl)->regions)
#define BLOB_REGION_AT(_bl, i)	((_bl)->regions[i])
#define BLOB_N_REGIONS(_bl)	((_bl)->nmb_regions)
#define BLOB_MOMENT(_bl,l,k)	((_bl)->moments[l][k])
#define BLOB_VEHICLE_AOI(_bl)	((_bl)->vehicle_aoi)
#define BLOB_NEXT(_bl)		((_bl)->next)
#define BLOB_PREVIOUS(_bl)	((_bl)->previous)
/** @} */

/** macro for getting pr_data attributes. */
#define PRD_TYPE(_prd)		((_prd)->type)
#define PRD_SUBTYPE(_prd)	((_prd)->subtype)
#define PRD_OCCL_TYPE(_prd)      ((_prd)->occl_type)

#define PRD_BBOX(_prd)		(&((_prd)->bbox))
#define PRD_3DBBOX(_prd)	(&((_prd)->_3Dbbox))
#define PRD_DDATA(_prd)		(&((_prd)->ddata))

#define PRD_XLEFT(_prd)		(RECT_XLEFT(PRD_BBOX(_prd)))
#define PRD_X(_prd)		(PRD_XLEFT(_prd))
#define PRD_XRIGHT(_prd)	(RECT_XRIGHT(PRD_BBOX(_prd)))
#define PRD_YTOP(_prd)		(RECT_YTOP(PRD_BBOX(_prd)))
#define PRD_Y(_prd)		(PRD_YTOP(_prd))
#define PRD_YBOTTOM(_prd)	(RECT_YBOTTOM(PRD_BBOX(_prd)))
#define PRD_WIDTH(_prd)		(RECT_WIDTH(PRD_BBOX(_prd)))
#define PRD_HEIGHT(_prd)	(RECT_HEIGHT(PRD_BBOX(_prd)))
#define PRD_XCENTER(_prd)	(PRD_XLEFT(_prd) + PRD_WIDTH(_prd)/2)
#define PRD_YCENTER(_prd)	(PRD_YTOP(_prd) + PRD_HEIGHT(_prd)/2)


#define PRD_3DBBOX_H(_prd)	  (PARALL_H(PRD_3DBBOX(_prd)))
#define PRD_3DBBOX_X_i(_prd, _i)  (PARALL_X_i(PRD_3DBBOX(_prd), _i))
#define PRD_3DBBOX_Y_i(_prd, _i)  (PARALL_Y_i(PRD_3DBBOX(_prd), _i))
#define PRD_3DBBOX_POINT_i(_prd, _i)  (PARALL_POINT_i(PRD_3DBBOX(_prd), _i))

#define PRD_3DBBOX_X2D_BASE_i(_prd, _i) (PARALL_X2D_BASE_i(PRD_3DBBOX(_prd), _i))
#define PRD_3DBBOX_Y2D_BASE_i(_prd, _i) (PARALL_Y2D_BASE_i(PRD_3DBBOX(_prd), _i))
#define PRD_3DBBOX_2D_BASE_POINT_i(_prd, _i) (PARALL_2D_BASE_POINT_i(PRD_3DBBOX(_prd), _i))

#define PRD_3DBBOX_X2D_H_i(_prd, _i) (PARALL_X2D_H_i(PRD_3DBBOX(_prd), _i))
#define PRD_3DBBOX_Y2D_H_i(_prd, _i) (PARALL_Y2D_H_i(PRD_3DBBOX(_prd), _i))
#define PRD_3DBBOX_2D_H_POINT_i(_prd, _i) (PARALL_2D_H_POINT_i(PRD_3DBBOX(_prd), _i))

#define PRD_3D_X(_prd)		((_prd)->position3d[0])
#define PRD_3D_Y(_prd)		((_prd)->position3d[1])
#define PRD_3D_Z(_prd)		((_prd)->position3d[2])
#define PRD_3D_POSITION(_prd)	((_prd)->position3d)

#define PRD_W(_prd)	((_prd)->w)
#define PRD_H(_prd)	((_prd)->h)
#define PRD_L(_prd)	((_prd)->l)

#define PRD_PW(_prd)	((_prd)->Pw)
#define PRD_PH(_prd)	((_prd)->Ph)
#define PRD_PL(_prd)	((_prd)->Pl)

#define PRD_RW(_prd)	((_prd)->Rw)
#define PRD_RH(_prd)	((_prd)->Rh)
#define PRD_RL(_prd)	((_prd)->Rl)

#define PRD_R(_prd)	((_prd)->R)
#define PRD_PR(_prd)	((_prd)->PR)
#define PRD_P(_prd)	((_prd)->P)
#define PRD_DP(_prd)	((_prd)->DP)


#define PRD_ALPHA(_prd)	((_prd)->alpha)

#define PRD_SUBTYPES_LIST(_prd) ((_prd)->subtypes_list)

#define PRD_NEXT(_prd)		((_prd)->next)
#define PRD_PREVIOUS(_prd)	((_prd)->previous)
/** @} */

/** macro for checking if two blobs are overlaped. */
#define B_OVERLAPPED(b1,b2)     (((MAX(BLOB_XLEFT(b1),BLOB_XLEFT(b2)) <=    \
                                   MIN(BLOB_XRIGHT(b1),BLOB_XRIGHT(b2))) && \
                                  (MAX(BLOB_YTOP(b1),BLOB_YTOP(b2))   <=    \
                                   MIN(BLOB_YBOTTOM(b1),BLOB_YBOTTOM(b2)))) \
                                 ? 1 : 0)

/**
 * Allocate the memory for new blob.
 * @return address of new blob, NULL if failed.
 * @note Allocate for its bounding box too.
 */
blob_t
blob_new();


/**
 * Free a blob.
 * @param blob blob to free.
 * @note free its bounding box too.
 */
void
blob_free(blob_t blob);

/**
 * Free blob's allocated lists.
 * @param blob blob to clean.
 */
void
blob_clean(blob_t blob);

/**
 * Free a blob list. blob_free() will be called for each blob in list.
 * @param list blob list to free.
 */
void
free_blob_list(blob_t list);


/**
 * Copy a blob, new blob will be created.
 * @param blob source blob.
 * @return address of new blob.
 * @note two blob will contain the same data except address of bounding box.
 */
blob_t
copy_blob(blob_t blob);


/**
 * Integral copy of a blob, new blob will be created. It also copies list of possible object types (prdata_t lists).
 * @param blob source blob.
 * @return address of new blob.
 * @note two blob will contain the same data except address of bounding box.
 */
blob_t
copy_blob_with_lists(blob_t blob);


/**
 * Copy a blob list, new blob will be created.
 * @param blob source blob.
 * @return address of new blob.
 */
blob_t
copy_blob_list(blob_t blob);

/**
 * Copy a blob, new blob is not created.
 * @param blob1 source blob.
 * @param blob2 destination blob
 * @return address of new blob.
 * @note two blob will contain the same data except address of bounding box.
 */
void
copy_blob(blob_t blob1, blob_t blob2);


/**
 * Copy blob data, but preserve connections and id of blob2, new blob is not created.
 * @param blob1 source blob.
 * @param blob2 destination blob
 * @return address of new blob.
 * @note two blob will contain the same data except address of bounding box.
 */
void
copy_blob_data(blob_t blob1, blob_t blob2);


/**
 * Test if blob is occluded
 * @return 1 if blob is occluded
 * @return 0 else
 **/
int IsBlobOccluted(blob_t pBlob);

/**
 * Merge two blob, result is stored in blob1.
 * @param blob1 the first blob.
 * @param blob2 the second blob.
 */
void merge_blob(blob_t blob1, blob_t blob2);


/**
 * Merge two blob, new blob will be created.
 * @param blob1 the first blob.
 * @param blob2 the second blob.
 * @return address of new blob.
 */
blob_t
merge_blob(blob_t blob1,
	   blob_t blob2,
	   struct scene_model *pTp_sm_node);


/**
 * Absorb a blob, like merge_blob() but not create new blob.
 * @param blob1 the first blob.
 * @param blob2 the second blob that will be absorb into blob1.
 * @return address of new blob.
 * @note blob1 attributs will be updated. blob2 not changed.
 */
blob_t
absorb_blob(blob_t blob1,
			blob_t blob2,
			struct scene_model *pTp_sm_node);

/**
 * Absorb a blob, like merge_blob() but not create new blob and do not recompute 3D parameters
 * @param blob1 the first blob.
 * @param blob2 the second blob that will be absorb into blob1.
 * @return address of new blob.
 * @note blob1 attributs will be updated. blob2 not changed.
 */
blob_t
absorb_blob_no3d(blob_t blob1,
			blob_t blob2,
			struct scene_model *pTp_sm_node);

/**
 * Test if two blobs have been extracted from one source blob in separation
 * step.
 * @param blob1 the first blob.
 * @param blob2 the second blob.
 * @return 1 if two blobs have been extracted from one source blob, otherwise 0.
 */
int
blob_same_father(blob_t blob1,
				 blob_t blob2);

/**
 * Get the biggest blob (size 2D) of a blob list.
 * @param blobs blob list.
 */
blob_t
blob_get_biggest2D(blob_t blobs);


/**
 * Remove a blob from blob list.
 * @param blob blob to be remove.
 * @param list blob list.
 */
blob_t
remove_blob(blob_t blob,
	    blob_t list);

/**
 * Get number of blobs in list by going through the list.
 * @param list blob list.
 */
int
blob_list_number(blob_t list);


/**
 * Compute 3D parameters (W3D, H3D, ...) for a blob.
 * @param bl blob.
 */
int
compute_3d_parameters_blob(struct scene_model *pTp_sm_node, blob_t bl);

/**
 * Gives back the blob type corresponding to a given string
 * @param name string
 */
blob_type_t
get_type_from_name(const char *name);

/**
 * Gives back the blob name corresponding to a given type
 * @param type integer
 */
std::string
get_name_from_type(blob_type_t type);

/**
 * Gives back the blob subtype corresponding to a given string
 * @param name string
 */
blob_subtype_t
get_subtype_from_name(const char *name);

/**
 * Gives back the blob name corresponding to a given subtype
 * @param type integer
 */
std::string
get_name_from_subtype(blob_subtype_t type);

/**
 * Gives back the blob type corresponding to a given string
 * @param name string
 */
blob_occl_type
get_occl_type_from_name(const char *name);

/**
 * Gives back the name corresponding to a given type of occlusion
 * @param type integer
 */
std::string
get_occl_name_from_type(blob_occl_type octype);
std::string
get_occl_name_from_type_detailed(blob_occl_type octype);

/**
 * Compute compute the overlap percentage between two blobs
 * \param blobs: bl and b2.
 * \return the overlap percentage between the two blols, output value in range [0...100]
 */
double
compute_overlap_ratio_blob(blob_t b1, blob_t b2);

/**
 * Compute compute the overlap percentage between two blobs, blob given as first parameter is
 * always taken as reference.
 * \param blobs: bl and b2.
 * \return the overlap percentage between the two blols, output value in range [0...100]
 */
double
compute_overlap_ratio_blob_strict(blob_t b1, blob_t b2);

double is_blob1_overlap_blob2(blob_t b1, blob_t b2);

int is_blob1_inside_blob2(blob_t b1, blob_t b2);

int isPixBelongsToBlob(blob_t b, int x, int y);

//std::list< std::tr1::shared_ptr < blob_t > > whichBlobsPixelBelongsTo(blob_t blobs, int x, int y);

bool isBlobOcculted(blob_t);
bool isBlobTopOcculted(blob_t);
bool isBlobBottomOcculted(blob_t);
bool isBlobHuman(blob_t);
bool isBlobAVehicle(blob_t);
bool isBlobARealObject(blob_t);

/**
 * Allocate the memory for new prdata structure.
 * @return address of new prdata, NULL if failed.
 */
prdata_t
prdata_new();

/**
 * Free a prdata structure.
 * @param prd prdata structure to be freed.
 */
void
prdata_free(prdata_t prd);

/**
 * Free a prd list. prdata_free() will be called for each prdata structure in list.
 * @param list prd list to be freed.
 */
void
free_prd_list(prdata_t list);

/**
 * Copy a prdata structure, new prdata structure will be created.
 * @param prd source prdata structure.
 * @return address of new prdata structure.
  */
prdata_t
copy_prdata(prdata_t prd);

/**
 * Copy a prdata structure, new prdata structure is not created.
 * @param prd1 source prdata structure.
 * @param prd2 destination prdata structure.
 */
void
copy_prdata(prdata_t prd1, prdata_t prd2);

/**
 * Copy a prdata list, new prdata will be created.
 * @param prd source prdata structure list.
 * @return address of new prdata structure list.
 */
prdata_t
copy_prdata_list(prdata_t list);

/**
 * Copy the prdata information to a blob.
 * @param prd source prdata structure.
 * @param blob destination blob structure.
 */
void
copy_prd_to_blob(prdata_t prd, blob_t blob);

/**
 * Copy the blob information to a prdata structure.
 * @param blob source blob structure.
 * @param prd destination prdata structure.
 */
void
copy_blob_to_prd(blob_t blob, prdata_t prd);

#if 0
bool IsblobOnOneAOIGround(blob_t i_blob, wdef_aoi_t i_pAoi);

bool isBlobOnAOIGround(blob_t blob, scene_model_t pTp_sm_node);
#endif


int get_blob_3D_bbox(blob_t blob, struct scene_model *pTp_sm_node, double h, double alpha);

int compute_blob_3Dbbox_normal_case(blob_t blob, struct scene_model *pTp_sm_node, double h, double alpha, double V[4][4], int limits[4][4], int nlimits[4], int eq_relations[4][4], int in_height[4]);

int compute_blob_3Dbbox_special_cases(blob_t blob, struct scene_model *pTp_sm_node, double h, double alpha, double V[4][4], int limits[4][4], int nlimits[4], int eq_relations[4][4], int in_height[4]);

void set_in_height_given_position(int position, int *in_height);


int get_blob_position_relative_to_camera(blob_t blob, struct scene_model *pTp_sm_node);

int get_blob_position_relative_to_camera_old(blob_t blob, struct scene_model *pTp_sm_node);

int get_rect_position_relative_to_camera(rect_t rectangle, struct scene_model *pTp_sm_node);

int get_rect_position_relative_to_camera_old(rect_t rectangle, struct scene_model *pTp_sm_node);

void set_real_3D_position(parallelepiped_t _3Dbbox, double position3d[3]);

void set_estimated_3D_position(	struct scene_model* m_context, rectangle_t bbox, double position3d[3], int position);

void set_position_at_center_bottom(struct scene_model* context, rectangle_t bbox, double position3d[3]);

int same2DBlob(blob_t b1, blob_t b2);

blob_t vertical_split_blob(blob_t blob, int x2d);

blob_t horizontal_split_blob(blob_t blob, int y2d);

void get3DboundingBoxFromInitial3DPoint(scene_model *context, rectangle_t bb2D, parallelepiped_t bb3D, double x3D, double y3D, int point_index,
					double angle, int beta_direction, double w, double l, double h);

void set2Dand3Dbboxes(scene_model *context, double *x, double *y, rect_t bb2D, parallelepiped_t bb3D, double h);


void set_null_3D_prd(blob_t blob, prdata_t prd);
void set_null_3D_prd(prdata_t prd, blob_type_t type, blob_subtype_t subtype, blob_occl_type occtype, rectangle_t rect);

void copy_prdata_no_lists(prdata_t prd1, prdata_t prd2);
void copy_prdata_only_info(prdata_t prd1, prdata_t prd2);

} // end namespace vsip

#endif
