#ifndef _COMMON_H_
#define _COMMON_H_

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string>

#include "TimeStamp.h"  
#include "MathFunctions.h"

namespace vsip {

	struct polygon_2d;

	//==================
	// DEFINEs  SECTION
	//==================

	/**
	 * maximal number of possible detected blobs for every frame.
	 */
	#define SEGM_BLOBS_MAX_NUMBER           2000

	/**
	 * history size, use for tracking.
	 */
	#define MOBILE_HIST_SIZE   		 10 //15 //20

	/**
	 * maximal number of io_zone in context file
	 */
	#define MAX_IO_ZONES                    10

	#define MAX_DOOR_NUMBER  (5)

	#define HUMAN_MIN_AGE 7
	#define MAX_SENSOR 			20
	#define FRAME_TIME 			40
	//==================
	// MACROS  SECTION
	//==================

	/**
	 * Performs a modulo sum for mobile_history
	 */
	#ifndef H_MODULO
	#define H_MODULO(num)          hmodulo(num, MOBILE_HIST_SIZE)
	#endif

	/**
	 * Get max number.
	 */
	#ifndef MIN
	#define MIN(a,b)	std::min(a,b)
	#endif


	/**
	 * Get min number.
	 */
	#ifndef MAX
	#define MAX(a,b)	std::max(a,b)
	#endif


	/**
	 * Macro used to replace rint() because of portability problems.
	 */
	#ifndef RINT
	#define RINT(a)         math::roundInt(a)
	#endif

	/**
	 * Test ascending order.
	 */
	#ifndef ASCENDING
	#define ASCENDING(a, b, c, d) ((((a)>=(b))||((b)>=(c))||((c)>=(d))) ? 0 : 1)
	#endif

	#define SQUARE(a)               math::square(a)

	#define DISTANCE_2D(m1,m2)      ((int) RINT(sqrt((double)SQUARE( ((m1->getXRight()+m1->getXLeft())/2)   - \
																	 ((m2->getXRight()+m2->getXLeft())/2) ) + \
														   (double)SQUARE(  m1->getYBottom() - m2->getYBottom() ) )))

	#define LOST 1

	#define GREEK_PI 3.14159265358979323846

	#define NUMERIC_ERROR 1e-10

	//===========================
	// COMMON STRUCTURES  SECTION
	//===========================
	typedef struct CamStamp
	{
	  char                *camera_sector ;
	  char                *camera_area;
	  char                *camera_id;//C02,C10
	  int                 cam_number;//2 ,10
	  int                 index_cam;//camera index passing in run_algo function =camera_num.

	}CamStamp_st;

	typedef struct JpegHeader
	{
	  CamStamp_st  cam_stamp;
	  TimeStamp_st time_stamp;

	}JpegHeader_st;

	typedef struct D_relational_generic D_relational_generic_st;
	typedef D_relational_generic_st    *D_relational_generic_t;

	 /*
	 * Function definition
	 */

	void print_info_poly2d(struct polygon_2d * poly);


	int hmodulo(int m, int modulo);

	void init_TimeStamp_st(TimeStamp_st* pTS);
	void init_CamStamp_st(CamStamp_st* pCS);
	void init_JpegHeader_st(JpegHeader_st* pJH);

	bool atob(const std::string& i_s);

} // end namespace vsip

#endif /* _COMMONS_H_ */


