/* supDataPhysicalObject.h ---
 *
 * Author: %Author%
 */

/* Commentary:
 *
 */

/* Change log:
 *
 */

#ifndef SUPDATAPHYSICALOBJ_H
#define SUPDATAPHYSICALOBJ_H

//#include <dtkCore/dtkAbstractData.h>

#include "supAbstractData.h"

#include <opencv2/core/core.hpp>
#include <string>

#include "supCoreExport.h"
#include "boundingbox3d.h"
#include "skeletonStructure.h"
#include <tr1/memory>




class supDataPhysicalObjPrivate;

class supDataPhysicalObjectPrivate;


// /////////////////////////////////////////////////////////////////
// supDataPhysicalObject interface
// /////////////////////////////////////////////////////////////////

class SUPCORE_EXPORT supDataPhysicalObject : public supAbstractData
{
    Q_OBJECT

public:
             supDataPhysicalObject(void);
             supDataPhysicalObject(const supDataPhysicalObject& other);
    virtual ~supDataPhysicalObject(void);

public:
    virtual supDataPhysicalObject *clone(void); // If your class is pure virtual you need to set this method to pure virtual too! 

public:
    supDataPhysicalObject& operator = (const supDataPhysicalObject& other);

protected:
   virtual void copy(const dtkAbstractObject& other);

public:
   QString identifier(void) const;

public:
    // Add your own setter methods
    void setId( const int id); 

    void setPosition( const cv::Point3f & position );

    void setCentroidPosition( const cv::Point3f & position);

    void setHighestPosition( const cv::Point3f & position);

    void setLowestPosition( const cv::Point3f & position);

    void setBoundingBox2D( const cv::Rect & rect ) ;

    void setType( const std::string &type) ;

    void setBoundingBox3D( const BoundingBox3D &bbox3d) ;

    void setSkeleton( const SkeletonStructure & skeleton);

///////////for Baptiste //////////////////////////////
    //for the sake of optimization, pixels content will be swap -> can't be const
    void setPixels( std::list < cv::Point2i > & pixels);
    
    void setVoxels ( std::list < cv::Point3f > & voxels);
///////////for Baptiste //////////////////////////////

    // Try to sort in the alphabetical order

public:
    // Add your own getter methods
    int getId() const;
    
    cv::Point3f getPosition() const;

    cv::Point3f getCentroidPosition() const;

    cv::Point3f getHighestPosition() const;

    cv::Point3f getLowestPosition() const;

    cv::Point3f getHead3dPosition() const;

    cv::Point2i getHead2dPosition() const;

    const cv::Rect& getBoundingBox2D() const;

    std::string getType() const; 

    BoundingBox3D getBoundingBox3D() const; 


    SkeletonStructure getSkeleton( ) const;

    operator QVariant() const
        {
            return QVariant::fromValue(*this);
        }

///////////for Baptiste //////////////////////////////
    void getPixels( std::list < cv::Point2i > & pixels) const;
    
    void getVoxels( std::list < cv::Point3f > & voxels) const;
///////////for Baptiste //////////////////////////////

    // Try to sort in the alphabetical order

public:
    // Methods you want to overload from dtkAbstractData

   // Try to sort in the alphabetical order

public:
    // Miscellaneous methods

    // Try to sort in the alphabetical order

private:
    DTK_DECLARE_PRIVATE(supDataPhysicalObject)
    
};

typedef std::list< supDataPhysicalObject > PhysicalObjs;


Q_DECLARE_METATYPE(supDataPhysicalObject)  // If your class is pure virtual, you cannot register to QMetatype.
Q_DECLARE_METATYPE(supDataPhysicalObject*) // Hence, you need to comment theses lines.

// dpchau
typedef  std::tr1::shared_ptr< supDataPhysicalObject > SpPhysicalObject; // todo use qshared pointer
typedef  std::list< SpPhysicalObject >         SpPhysicalObjectList;

// todo remove the following definition
namespace sup{
    typedef  SpPhysicalObject     SpPhysicalObjectInterface;
    typedef  SpPhysicalObjectList SpPhysicalObjectInterfaceList;
}



#endif

